%CD2
% Numerical solution of 2-D singular perturbation problem for stationary
% convection-diffusion equation:
%  	- dw/dx - D*Laplacian(w) = q

% Copyright 2001 P. Wesseling
% This program and its subprograms may be freely used, modified and distributed
% under the GNU General Public License: http://www.gnu.org/copyleft/gpl.html

% Boundary conditions: 
%	x = 1: Dirichlet   x = 0: homogeneous or inhomogeneous Neumann
%	y = 0: Dirichlet   y = 1: homogeneous Neumann

% Central or upwind scheme for convection
% Cell-centered grid; uniform in x, locally refined in y

% Theory is given in Section 3.3 of
%	Computational Fluid Dynamics
%	Lecture Notes by P. Wesseling
%	Department of Applied Mathematical Analysis, ITS, TU Delft
% See http://ta.twi.tudelft.nl/nw/users/wesseling/

% This program generates Tables 3.1 and 3.2 in the Lecture Notes

% Functions called: exact_solution, fL, q,  
%		    rhs,  matrix1,  matrix2,  matrix3

clear all
		% .......................Input..............................
D = 0.001;	% Diffusion coefficient ( = 1/[Peclet number] )
nx = 8;		% Horizontal number of cells
nyc = 4;	% Vertical number of cells outside refinement zone
nyf = 32;	% Vertical number of cells inside refinement zone
central = 0;	% Enter 1 for central scheme or something else for upwind scheme
hom = 0;	% Neumann condition at outflow boundary: 1 for inhomogeneous
		% (exact) condition, or something else for homogeneous condition
th = 8;		% Parameter governing thickness of refinement zone 
		% thickness = sigma = th*sqrt(D)
		% .....................End of input......................

sigma = th*sqrt(D);	% Thickness of refinement zone

% Implementation of upwind scheme by artificial diffusion in x-direction
dx = 1/nx;
if central == 1		% Central scheme for convection
  Da = D;		% Artificially increased diffusion coefficient
else			% Upwind scheme for convection, implemented by
  Da = D + dx/2;	%   artificial viscosity; grid assumed uniform 
end			%   in x-direction!

%...............Definition of volume boundaries and grid points
%		Two-dimensional grid
 
%     x = 0    
% grid |---o---|---o---|---o---|---o---|		Grid in x-direction
%      1   1   2   2   3           J

%      |<---- sigma--->|                      
%     y=0    K1 cells  |        K2 cells      y=1
% grid |-o-|-o-|-o-|-o-|---o---|---o---|---o---|	Grid in y-direction
%      1 1 2 2 3 3         

J = nx; K1 = nyf; K2 = nyc; K = K1 + K2;  % In the lecture notes, J and K
					  % are called I and K respectively
					  
x = [1:J]'*dx - dx/2;		% Horizontal coordinates of cell centers

by = zeros(K+1,1);		% Vertical coordinates of cell boundaries
k = 1:K1+1;    by(k) = (k-1)*sigma/K1;
k = K1+2:K+1;  by(k) = sigma + (k-K1-1)*(1-sigma)/K2;

n = length(by); y = zeros(n-1,1);	% Vertical coordinates of cell centers
dy = zeros(n-1,1);			% Vertical mesh sizes 
y = 0.5*(by(1:n-1) + by(2:n)); 		dy = diff(by);
[DX,DY] = meshgrid(dx*ones(J,1),dy); 	[X,Y] = meshgrid(x,y);

%.................End of grid generation................................

rhs  		% Generation of right-hand side with elapsed time output

% Three implementations of matrix generation in order of increasing
% efficiency, with elapsed time output: 
matrix1	     	% Generation of matrix A1
matrix2		% Generation of matrix A2 
matrix3		% Generation of matrix A3 

tic		% Start clock	   
numerical_solution = A3\bb;
time = toc;	% Stop clock3
disp(['Solution time = ',num2str(time)])	% Screen output

% Screen output of error norms
error = numerical_solution - reshape(exact_solution(X,Y,D)',J*K,1); 
n = K*J;
disp(['norm1 = ',num2str(norm(error,1)/n)])
disp(['norm2 = ',num2str(norm(error,2)/sqrt(n))])
[norminf,m] = max(abs(error));
disp(['norminf = ',num2str(norminf),' at m = ',num2str(m)])

% ..................Solution profile at x = x(1) = dx/2....................
figure (1), clf, subplot(2,2,1), hold on, 
plot(numerical_solution(1:J:1 + (K-1)*J),y,'o')
yy = linspace(0,1,51);
plot(exact_solution(x(1)*ones(1,length(yy)),yy,D),yy)
title(['Profile at x(1)'],'fontsize',20)

% ..................Solution profile at y = y(kkk).......................
kkk = 1;
subplot(2,2,4), plot(x,numerical_solution(1+ (kkk-1)*J:1:kkk*J),'o')
hold on, plot(yy,exact_solution(yy, y(kkk)*ones(1,length(yy)), D))
title(['Profile at y(',num2str(kkk),')'],'fontsize',20)

% ...................Solution profile at x = x(J)..........................
subplot(2,2,2),  title(['Inflow profile'],'fontsize',20),  hold on
plot(numerical_solution(J:J:J*K),y,'o')
plot(exact_solution(x(J)*ones(1,length(yy)), yy, D),yy)

% ..................Contour plot.....................................
subplot(2,2,3), contour(x,y,reshape(numerical_solution,J,K)'), hold on
title(['Isolines'],'fontsize',20) 
% ...................Waterfall plot......................................
figure(2),clf, waterfall(x,y,reshape(numerical_solution,J,K)'), 
view(200,70), colormap([0 0 0]); %grid off





